function fish_user_key_bindings
    bind --key f1 "dial_insert 1"
    bind --key f2 "dial_insert 2"
    bind --key f3 "dial_insert 3"
    bind --key f4 "dial_insert 4"
    bind --key f5 "dial_insert 5"
    bind --key f6 "dial_insert 6"
    bind --key f7 "dial_insert 7"
    bind --key f8 "dial_insert 8"
    bind --key f9 "dial_insert 9"
    bind --key f10 "dial_insert 10"
    bind --key f11 "dial_insert 11"
    bind --key f12 "dial_insert 12"
    bind \e\[1\;2P "dial_cd 1"
    bind \e\[1\;2Q "dial_cd 2"
    bind \e\[1\;2R "dial_cd 3"
    bind \e\[1\;2S "dial_cd 4"
    bind \e\[15\;2~ "dial_cd 5"
    bind \e\[17\;2~ "dial_cd 6"
    bind \e\[18\;2~ "dial_cd 7"
    bind \e\[19\;2~ "dial_cd 8"
    bind \e\[20\;2~ "dial_cd 9"
    bind \e\[21\;2~ "dial_cd 10"
    bind \e\[23\;2~ "dial_cd 11"
    bind \e\[24\;2~ "dial_cd 12"
    bind \e\[1\;5P "dial_cd_ctrl 1"
    bind \e\[1\;5Q "dial_cd_ctrl 2"
    bind \e\[1\;5R "dial_cd_ctrl 3"
    bind \e\[1\;5S "dial_cd_ctrl 4"
    bind \e\[15\;5~ "dial_cd_ctrl 5"
    bind \e\[17\;5~ "dial_cd_ctrl 6"
    bind \e\[18\;5~ "dial_cd_ctrl 7"
    bind \e\[19\;5~ "dial_cd_ctrl 8"
    bind \e\[20\;5~ "dial_cd_ctrl 9"
    bind \e\[21\;5~ "dial_cd_ctrl 10"
end
